﻿-----------------------------------------------------------------------
-- AddOn namespace.
-----------------------------------------------------------------------
local ADDON_NAME, private = ...
local LibStub = _G.LibStub
local WardrobeAchiever = LibStub("AceAddon-3.0"):GetAddon("WardrobeAchiever")

local ACHIEVEMENTUI_WIDTH = 500;

local ACHIEVEMENTUI_MAX_LINES_COLLAPSED = 3;
local ACHIEVEMENTBUTTON_COLLAPSEDHEIGHT = 84;
local ACHIEVEMENTUI_MAXCONTENTWIDTH = 330;
local ACHIEVEMENT_FLAGS_ACCOUNT = 0x00020000;

function WardrobeAchiever:addAchievement(parentFrame, achievementID, index) 
	-- Extract achievement info
	local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch, wasEarnedByMe, earnedBy = GetAchievementInfo(achievementID)

	-- --------------------- --
	-- Achievement button
	-- --------------------- --
	local achievementFrame = _G.CreateFrame("Button", "achievementFrame"..id, parentFrame, "SecureActionButtonTemplate")
	achievementFrame:SetSize(ACHIEVEMENTUI_WIDTH, ACHIEVEMENTBUTTON_COLLAPSEDHEIGHT)
	achievementFrame:SetPoint("TOP", 0, -(ACHIEVEMENTBUTTON_COLLAPSEDHEIGHT * (index - 1)))
	achievementFrame:SetBackdrop({
		tile = true,
		edgeSize = 16,
		edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]]
	})
	achievementFrame:SetBackdropBorderColor(0.7,0.15,0.05,1);
	achievementFrame.Saturate = WA_AchievementButton_Saturate;
	achievementFrame.Desaturate = WA_AchievementButton_Desaturate;
	achievementFrame.Click = WA_AchievementButton_OnClick;
	achievementFrame.Update = WA_UpdateAchievement;
	achievementFrame.id = id
	
	achievementFrame:SetScript("OnClick", function(self)
		self:Click();
	end)
	achievementFrame:SetScript("OnEnter", function(self) 
		self.parentHighlight:Show();
	end)
	achievementFrame:SetScript("OnLeave", function(self) 
		if (not self.selected) then
			self.parentHighlight:Hide();
		end
	end)
	
	-- --------------------- --
	-- Textures
	-- --------------------- --
	achievementFrame.parentBackground = achievementFrame:CreateTexture("$parentBackground", "BACKGROUND")
	achievementFrame.parentBackground:SetTexture([[Interface\AchievementFrame\UI-Achievement-Parchment-Horizontal]])
	achievementFrame.parentBackground:SetPoint("TOPLEFT",3,-3)
	achievementFrame.parentBackground:SetPoint("BOTTOMRIGHT",-3,3)

	local parentBottomLeftTsunami = achievementFrame:CreateTexture("$parentBottomLeftTsunami", "BORDER")
	parentBottomLeftTsunami:SetTexture([[Interface\AchievementFrame\UI-Achievement-Tsunami-Corners]])
	parentBottomLeftTsunami:SetSize(32, 32)
	parentBottomLeftTsunami:SetPoint("BOTTOMLEFT",-2,-2)
	parentBottomLeftTsunami:SetVertexColor(1,1,1,.2)
	parentBottomLeftTsunami:SetTexCoord(0,.5,0,1)

	local parentBottomRightTsunami = achievementFrame:CreateTexture("$parentBottomRightTsunami", "BORDER")
	parentBottomRightTsunami:SetTexture([[Interface\AchievementFrame\UI-Achievement-Tsunami-Corners]])
	parentBottomRightTsunami:SetSize(32, 32)
	parentBottomRightTsunami:SetPoint("BOTTOMRIGHT",2,-2)
	parentBottomRightTsunami:SetVertexColor(1,1,1,.2)
	parentBottomRightTsunami:SetTexCoord(.5,1,0,1)

	local parentTopLeftTsunami = achievementFrame:CreateTexture("$parentTopLeftTsunami", "BORDER")
	parentTopLeftTsunami:SetTexture([[Interface\AchievementFrame\UI-Achievement-Tsunami-Corners]])
	parentTopLeftTsunami:SetSize(32, 32)
	parentTopLeftTsunami:SetPoint("TOPLEFT",-2,-20)
	parentTopLeftTsunami:SetVertexColor(1,1,1,.1)
	parentTopLeftTsunami:SetTexCoord(1,.5,1,0)

	local parentTopRightTsunami = achievementFrame:CreateTexture("$parentTopRightTsunami", "BORDER")
	parentTopRightTsunami:SetTexture([[Interface\AchievementFrame\UI-Achievement-Tsunami-Corners]])
	parentTopRightTsunami:SetSize(32, 32)
	parentTopRightTsunami:SetPoint("TOPRIGHT",2,-20)
	parentTopRightTsunami:SetVertexColor(1,1,1,.1)
	parentTopRightTsunami:SetTexCoord(.5,0,1,0)

	local parentBottomTsunami1 = achievementFrame:CreateTexture("$parentBottomTsunami1", "BORDER")
	parentBottomTsunami1:SetTexture([[Interface\AchievementFrame\UI-Achievement-Borders]])
	parentBottomTsunami1:SetSize(370, 16)
	parentBottomTsunami1:SetPoint("BOTTOMLEFT", parentBottomLeftTsunami, "BOTTOMRIGHT", 0, 3)
	parentBottomTsunami1:SetPoint("BOTTOMRIGHT", parentBottomRightTsunami, "BOTTOMLEFT")
	parentBottomTsunami1:SetVertexColor(1,1,1,.35)
	parentBottomTsunami1:SetTexCoord(0, 0.72265, 0.51953125, 0.58203125)

	local parentTopTsunami1 = achievementFrame:CreateTexture("$parentTopTsunami1", "BORDER")
	parentTopTsunami1:SetTexture([[Interface\AchievementFrame\UI-Achievement-Borders]])
	parentTopTsunami1:SetSize(370, 16)
	parentTopTsunami1:SetPoint("TOPLEFT", parentTopLeftTsunami, "TOPRIGHT", 0, -3)
	parentTopTsunami1:SetPoint("TOPRIGHT", parentTopRightTsunami, "TOPLEFT")
	parentTopTsunami1:SetVertexColor(1,1,1,0.3)
	parentTopTsunami1:SetTexCoord(0.72265, 0, 0.58203125, 0.51953125)

	achievementFrame.parentTitleBackground = achievementFrame:CreateTexture("$parentTitleBackground", "ARTWORK")
	achievementFrame.parentTitleBackground:SetTexture([[Interface\AchievementFrame\UI-Achievement-Borders]])
	achievementFrame.parentTitleBackground:SetSize(0, 24)
	achievementFrame.parentTitleBackground:SetPoint("TOPLEFT", 5, -4)
	achievementFrame.parentTitleBackground:SetPoint("TOPRIGHT", -5, -4)
	achievementFrame.parentTitleBackground:SetVertexColor(1,1,1,.8)
	achievementFrame.parentTitleBackground:SetTexCoord(0, 0.9765625, 0.66015625, 0.73828125)

	achievementFrame.parentGlow = achievementFrame:CreateTexture("$parentGlow", "ARTWORK")
	achievementFrame.parentGlow:SetTexture([[Interface\AchievementFrame\UI-Achievement-Borders]])
	achievementFrame.parentGlow:SetSize(10, 64)
	achievementFrame.parentGlow:SetPoint("TOPLEFT", achievementFrame.parentTitleBackground, "BOTTOMLEFT", 0, 4)
	achievementFrame.parentGlow:SetPoint("RIGHT", 0, 4)
	achievementFrame.parentGlow:SetTexCoord(0, 1, 0.00390625, 0.25390625)

	achievementFrame.parentLabel = achievementFrame:CreateFontString("$parentLabel", "OVERLAY", "GameFontHighlightMedium")
	achievementFrame.parentLabel:SetText(name) -- ##TEXT. Achievement name
	achievementFrame.parentLabel:SetSize(300, 20)
	achievementFrame.parentLabel:SetPoint("TOP", achievementFrame.parentTitleBackground, "TOP", 0, 0)
	
	achievementFrame.parentRewardBackground = achievementFrame:CreateTexture("$parentRewardBackground", "ARTWORK")
	achievementFrame.parentRewardBackground:SetTexture([[Interface\AchievementFrame\UI-Achievement-Reward-Background]])
	achievementFrame.parentRewardBackground:SetSize(384, 24)
	achievementFrame.parentRewardBackground:SetPoint("BOTTOMLEFT", 5, -2)
	achievementFrame.parentRewardBackground:SetPoint("BOTTOMRIGHT", -5, 5)
	achievementFrame.parentRewardBackground:SetTexCoord(0, .69, 0, .75)

	local parentReward = achievementFrame:CreateFontString("$parentReward", "OVERLAY", "GameFontNormalSmall")
	parentReward:SetSize(400, 20)
	parentReward:SetPoint("TOP", achievementFrame.parentRewardBackground, "TOP", 0, 1)
		
	if (rewardText == "") then
		achievementFrame.parentRewardBackground:Hide();
		parentReward:Hide();
	else
		parentReward:SetText(rewardText);  -- ##TEXT.reward
		parentReward:Show();
		achievementFrame.parentRewardBackground:Show();
		if (completed) then
			achievementFrame.parentRewardBackground:SetVertexColor(1, 1, 1);
		else
			achievementFrame.parentRewardBackground:SetVertexColor(0.35, 0.35, 0.35);
		end
	end	
	
	achievementFrame.parentDescription = achievementFrame:CreateFontString("$parentDescription", "OVERLAY", "AchievementDescriptionFont")
	achievementFrame.parentDescription:SetText(description) -- ##ICON. Achievement description
	achievementFrame.parentDescription:SetPoint("TOP", 0, -30)
	local _, fontHeight = achievementFrame.parentDescription:GetFont();
	achievementFrame.parentDescription:SetSize(ACHIEVEMENTUI_MAXCONTENTWIDTH, fontHeight * ACHIEVEMENTUI_MAX_LINES_COLLAPSED)
	
	-- -------------------------------- --
	-- Inner frames / highlight
	-- -------------------------------- --
	achievementFrame.parentHighlight = _G.CreateFrame("Frame", "$parentHighlight", achievementFrame)
	achievementFrame.parentHighlight:SetAllPoints()
	achievementFrame.parentHighlight:Hide()
	achievementFrame.parentHighlight:EnableMouse(false)

	local parentTopLeft = achievementFrame.parentHighlight:CreateTexture("$parentTopLeft", "OVERLAY")
	parentTopLeft:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-ReputationBar-Highlight]])
	parentTopLeft:SetSize(16, 16)
	parentTopLeft:SetPoint("TOPLEFT", -1, 2)
	parentTopLeft:SetTexCoord(0.06640625, 0, 0.4375, 0.65625)
	parentTopLeft:SetBlendMode("ADD")

	local parentBottomLeft = achievementFrame.parentHighlight:CreateTexture("$parentBottomLeft", "OVERLAY")
	parentBottomLeft:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-ReputationBar-Highlight]])
	parentBottomLeft:SetSize(16, 16)
	parentBottomLeft:SetPoint("BOTTOMLEFT", -1, -2)
	parentBottomLeft:SetTexCoord(0.06640625, 0, 0.65625, 0.4375)
	parentBottomLeft:SetBlendMode("ADD")

	local parentTopRight = achievementFrame.parentHighlight:CreateTexture("$parentTopRight", "OVERLAY")
	parentTopRight:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-ReputationBar-Highlight]])
	parentTopRight:SetSize(16, 16)
	parentTopRight:SetPoint("TOPRIGHT", 1, 2)
	parentTopRight:SetTexCoord(0, 0.06640625, 0.4375, 0.65625)
	parentTopRight:SetBlendMode("ADD")

	local parentBottomRight = achievementFrame.parentHighlight:CreateTexture("$parentBottomRight", "OVERLAY")
	parentBottomRight:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-ReputationBar-Highlight]])
	parentBottomRight:SetSize(16, 16)
	parentBottomRight:SetPoint("BOTTOMRIGHT", 1, -2)
	parentBottomRight:SetTexCoord(0, 0.06640625, 0.65625, 0.4375)
	parentBottomRight:SetBlendMode("ADD")
		
	local parentTop = achievementFrame.parentHighlight:CreateTexture("$parentTop", "OVERLAY")
	parentTop:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-ReputationBar-Highlight]])
	parentTop:SetSize(16, 16)
	parentTop:SetPoint("TOPLEFT", parentTopLeft, "TOPRIGHT")
	parentTop:SetPoint("BOTTOMRIGHT", parentTopRight, "BOTTOMLEFT")
	parentTop:SetTexCoord(0, 0.015, 0.4375, 0.65625)
	parentTop:SetBlendMode("ADD")					
			
	local parentBottom = achievementFrame.parentHighlight:CreateTexture("$parentBottom", "OVERLAY")
	parentBottom:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-ReputationBar-Highlight]])
	parentBottom:SetSize(16, 16)
	parentBottom:SetPoint("TOPLEFT", parentBottomLeft, "TOPRIGHT")
	parentBottom:SetPoint("BOTTOMRIGHT", parentBottomRight, "BOTTOMLEFT")
	parentBottom:SetTexCoord(0, 0.015, 0.65625, 0.4375)
	parentBottom:SetBlendMode("ADD")		
				
	local parentLeft = achievementFrame.parentHighlight:CreateTexture("$parentLeft", "OVERLAY")
	parentLeft:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-ReputationBar-Highlight]])
	parentLeft:SetSize(16, 16)
	parentLeft:SetPoint("TOPLEFT", parentTopLeft, "BOTTOMLEFT")
	parentLeft:SetPoint("BOTTOMRIGHT", parentBottomLeft, "TOPRIGHT")
	parentLeft:SetTexCoord(0.06640625, 0, 0.65625, 0.6)
	parentLeft:SetBlendMode("ADD")						
								
	local parentRight = achievementFrame.parentHighlight:CreateTexture("$parentRight", "OVERLAY")
	parentRight:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-ReputationBar-Highlight]])
	parentRight:SetSize(16, 16)
	parentRight:SetPoint("TOPLEFT", parentTopRight, "BOTTOMLEFT")
	parentRight:SetPoint("BOTTOMRIGHT", parentBottomRight, "TOPRIGHT")
	parentRight:SetTexCoord(0, 0.06640625, 0.65625, 0.6)
	parentRight:SetBlendMode("ADD")		

	-- -------------------------------- --
	-- Inner frames / achievement icon
	-- -------------------------------- --
	achievementFrame.parentIcon = _G.CreateFrame("Frame", "parentIcon", achievementFrame)
	achievementFrame.parentIcon:SetSize(60, 60)
	achievementFrame.parentIcon:SetPoint("TOPLEFT", 8, -9)
	achievementFrame.parentIcon.Desaturate = WA_AchievementIcon_Desaturate;
	achievementFrame.parentIcon.Saturate = WA_AchievementIcon_Saturate;
	
	achievementFrame.parentIcon.parentBling = achievementFrame.parentIcon:CreateTexture("$parentBling", "BORDER")
	achievementFrame.parentIcon.parentBling:SetTexture([[Interface\AchievementFrame\UI-Achievement-Bling]])
	achievementFrame.parentIcon.parentBling:SetSize(116, 116)
	achievementFrame.parentIcon.parentBling:SetPoint("CENTER", -1, 1)
	achievementFrame.parentIcon.parentBling:Hide()	

	achievementFrame.parentIcon.parentTexture = achievementFrame.parentIcon:CreateTexture("$parentTexture", "ARTWORK")
	achievementFrame.parentIcon.parentTexture:SetTexture(icon) -- ##ICON. Achievement icon
	achievementFrame.parentIcon.parentTexture:SetSize(50, 50)
	achievementFrame.parentIcon.parentTexture:SetPoint("CENTER", 0, 3)

	achievementFrame.parentIcon.parentOverlay = achievementFrame.parentIcon:CreateTexture("$parentOverlay", "ARTWORK")
	achievementFrame.parentIcon.parentOverlay:SetTexture([[Interface\AchievementFrame\UI-Achievement-IconFrame]])
	achievementFrame.parentIcon.parentOverlay:SetSize(72, 72)
	achievementFrame.parentIcon.parentOverlay:SetPoint("CENTER", -1, 2)
	achievementFrame.parentIcon.parentOverlay:SetTexCoord(0, 0.5625, 0, 0.5625)

	-- -------------------------------- --
	-- Inner frames / achievement shield
	-- -------------------------------- --
	achievementFrame.parentShield = _G.CreateFrame("Button", "$parentShield", achievementFrame)
	achievementFrame.parentShield:SetSize(64, 64)
	achievementFrame.parentShield:SetPoint("TOPRIGHT", -6, 0)
	achievementFrame.parentShield.Desaturate = WA_AchievementShield_Desaturate;
	achievementFrame.parentShield.Saturate = WA_AchievementShield_Saturate;

	achievementFrame.parentShield.parentIcon = achievementFrame.parentShield:CreateTexture("$parentIcon", "BACKGROUND")
	achievementFrame.parentShield.parentIcon:SetTexture([[Interface\AchievementFrame\UI-Achievement-Shields]])
	achievementFrame.parentShield.parentIcon:SetSize(66, 64)
	achievementFrame.parentShield.parentIcon:SetPoint("TOPRIGHT", 0, -6)
	achievementFrame.parentShield.parentIcon:SetTexCoord(0, .5, 0, 0.5)

	achievementFrame.parentShield.parentPoints = achievementFrame.parentShield:CreateFontString("$parentPoints", "OVERLAY", "AchievementPointsFont")
	achievementFrame.parentShield.parentPoints:SetText(points) -- ##TEXT. Shield points
	achievementFrame.parentShield.parentPoints:SetVertexColor(1, 1, 1);
	achievementFrame.parentShield.parentPoints:SetSize(42, 16)
	achievementFrame.parentShield.parentPoints:SetPoint("TOPRIGHT", -13, -26)

	achievementFrame.parentShield.parentDateCompleted = achievementFrame.parentShield:CreateFontString("$parentDateCompleted", "OVERLAY", "AchievementDateFont")
	achievementFrame.parentShield.parentDateCompleted:SetSize(100, 14)
	achievementFrame.parentShield.parentDateCompleted:SetPoint("TOP", achievementFrame.parentShield, "BOTTOM", -2, 6)
	achievementFrame.parentShield.parentDateCompleted:SetJustifyH("CENTER")
	achievementFrame.parentShield.parentDateCompleted:Hide()

	achievementFrame.parentShield:SetScript("OnEnter", function(self) 
		local parent = self:GetParent();
		local func = parent:GetScript("OnEnter");
		if (func) then
			func(parent);
		end
	end)
	achievementFrame.parentShield:SetScript("OnLeave", function(self) 
		local parent = self:GetParent();
		local func = parent:GetScript("OnLeave");
		if (func) then
			func(parent);
		end
	end)
	achievementFrame.parentShield:SetScript("OnClick", function(self) 
		local parent = self:GetParent();
		local func = parent:GetScript("OnClick");
		if (func) then
			func(parent);
		end
	end)

	-- -------------------------------- --
	-- Inner frames / achievement objectives
	-- -------------------------------- --
	achievementFrame.parentObjectives = _G.CreateFrame("Frame", "$parentObjectives", achievementFrame)
	achievementFrame.parentObjectives:SetPoint("TOP", achievementFrame.parentDescription, "BOTTOM", 0, -8)
	achievementFrame.parentObjectives:SetPoint("LEFT",  achievementFrame.parentIcon, "RIGHT", 8, 0)
	achievementFrame.parentObjectives:SetPoint("RIGHT", achievementFrame.parentShield, "LEFT", -10, 0)
	achievementFrame.parentObjectives:SetPoint("BOTTOM", achievementFrame.parentRewardBackground, "TOP", 00, 0)
	
	-- -------------------------------- --
	-- Completed or nor completed
	-- -------------------------------- --
	local saturatedStyle;
	if (bit.band(flags, ACHIEVEMENT_FLAGS_ACCOUNT) == ACHIEVEMENT_FLAGS_ACCOUNT) then
		achievementFrame.accountWide = true;
		saturatedStyle = "account";
	else
		achievementFrame.accountWide = nil;
		saturatedStyle = "normal";
	end
	
	if (completed or wasEarnedByMe) then
		achievementFrame.completed = true;
		achievementFrame.parentShield.parentDateCompleted:SetText(FormatShortDate(day, month, year));  -- ##TEXT. Achievement accomplishment date
		achievementFrame.parentShield.parentDateCompleted:Show();
		if (achievementFrame.saturatedStyle ~= saturatedStyle) then
			achievementFrame:Saturate();
		end
	else
		achievementFrame.completed = nil;
		achievementFrame.parentShield.parentDateCompleted:Hide();
		achievementFrame:Desaturate();
	end
	
	parentFrame.buttons[index] = achievementFrame;
end

function WA_UpdateAchievement(self) 
	-- Extract achievement info
	local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch, wasEarnedByMe, earnedBy = GetAchievementInfo(self.id)
	
	if (rewardText ~= "") then
		if (completed) then
			self.parentRewardBackground:SetVertexColor(1, 1, 1);
		else
			self.parentRewardBackground:SetVertexColor(0.35, 0.35, 0.35);
		end
	end	

	if (completed or wasEarnedByMe) then
		self.completed = true;
		self.parentShield.parentDateCompleted:SetText(FormatShortDate(day, month, year));  -- ##TEXT. Achievement accomplishment date
		self.parentShield.parentDateCompleted:Show();
		self:Saturate();
	else
		self.completed = nil;
		self.parentShield.parentDateCompleted:Hide();
		self:Desaturate();
	end
end

function WA_AchievementButton_Saturate(self)
	self.parentBackground:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Parchment-Horizontal");
	if (self.accountWide) then
		self.parentTitleBackground:SetTexture("Interface\\AchievementFrame\\AccountLevel-AchievementHeader");
		self.parentTitleBackground:SetTexCoord(0, 1, 0, 0.375);
		self:SetBackdropBorderColor(0.129, 0.671, 0.875, 1);
		self.saturatedStyle = "account";
	else
		self.parentTitleBackground:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Borders");
		self.parentTitleBackground:SetTexCoord(0, 1, 0.66015625, 0.73828125);
		self:SetBackdropBorderColor(0.7, 0.15, 0.05, 1);
		self.saturatedStyle = "normal";
	end
	
	self.parentShield.parentPoints:SetVertexColor(1, 1, 1);
	self.parentGlow:SetVertexColor(1.0, 1.0, 1.0);
	self.parentIcon:Saturate();
	self.parentShield:Saturate();
	self.parentRewardBackground:SetVertexColor(1, .82, 0);
	self.parentLabel:SetVertexColor(1, 1, 1);
	self.parentDescription:SetTextColor(0, 0, 0, 1);
	self.parentDescription:SetShadowOffset(0, 0);
end

function WA_AchievementButton_Desaturate(self)
	self.saturatedStyle = nil;
	self.parentBackground:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Parchment-Horizontal-Desaturated");
	if (self.accountWide) then
		self.parentTitleBackground:SetTexture("Interface\\AchievementFrame\\AccountLevel-AchievementHeader");
		self.parentTitleBackground:SetTexCoord(0, 1, 0.40625, 0.78125);
	else
		self.parentTitleBackground:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Borders");
		self.parentTitleBackground:SetTexCoord(0, 1, 0.91796875, 0.99609375);
	end
		
	self.parentGlow:SetVertexColor(.22, .17, .13);
	self.parentIcon:Desaturate();
	self.parentShield:Desaturate();
	self.parentShield.parentPoints:SetVertexColor(.65, .65, .65);
	self.parentRewardBackground:SetVertexColor(.8, .8, .8);
	self.parentLabel:SetVertexColor(.65, .65, .65);
	self.parentDescription:SetTextColor(1, 1, 1, 1); 
	self.parentDescription:SetShadowOffset(1, -1);
	self:SetBackdropBorderColor(.5, .5, .5);
end

function WA_AchievementShield_Desaturate(self)
	self.parentIcon:SetTexCoord(.5, 1, 0, 0.5);
end

function WA_AchievementShield_Saturate(self)
	self.parentIcon:SetTexCoord(0, .5, 0, 0.5);
end

function WA_AchievementIcon_Desaturate(self)
	self.parentBling:SetVertexColor(.6, .6, .6, 1);
	self.parentOverlay:SetVertexColor(.75, .75, .75, 1);
	self.parentTexture:SetVertexColor(.55, .55, .55, 1);
end

function WA_AchievementIcon_Saturate(self)
	self.parentBling:SetVertexColor(1, 1, 1, 1);
	self.parentOverlay:SetVertexColor(1, 1, 1, 1);
	self.parentTexture:SetVertexColor(1, 1, 1, 1);
end

function WA_AchievementButton_OnClick(self)
	WA_AchievementFrameAchievements_ClearSelection(self:GetParent());
	WA_AchievementFrameAchievements_SelectButton(self);
end

function WA_AchievementFrameAchievements_ClearSelection(parent)
	table.foreach(parent.buttons, function(k,button)
		if (not button:IsMouseOver()) then
			button.parentHighlight:Hide();
		end
		button.selected = nil;
	end)
	
	parent.achievementSelected = nil;
end

function WA_AchievementFrameAchievements_SelectButton(button)
	button:GetParent().achievementSelected = button.id;
	button.selected = true;
	button.parentHighlight:Show();
	
	local setID = WardrobeAchiever:getAchiSetID(button.id)
	button:GetParent():GetParent():GetParent().collectionsFrame:DisplaySet(setID)
end